/* -*- mode: c++; c-basic-offset: 4; -*- */
//
//    Build a channel list from a specified frame
#include "framecpp/FrameH.hh"
#include "framecpp/FrVect.hh"
// #include "framecpp/Dimension.hh"
#include "framecpp/FrRawData.hh"
#include "framecpp/FrAdcData.hh"
#include "framecpp/FrProcData.hh"
#include "framecpp/Common/IOStream.hh"
#include "framecpp/IFrameStream.hh"
#include "general/fstream.hh"
#include <string>
#include <iostream>
#include <iomanip>
#include <map>

//======================================  Define pointer types
#include "ldas/ldasconfig.hh"
#if LDAS_VERSION_NUMBER > 119010
#define FCPP_SHARED_PTRS 1
#include "general/SharedPtr.hh"
typedef General::SharedPtr<FrameCPP::FrameH>     frame_pointer;
typedef General::SharedPtr<FrameCPP::FrRawData>  frrawdata_pointer;
typedef General::SharedPtr<FrameCPP::FrAdcData>  fradcdata_pointer;
typedef General::SharedPtr<FrameCPP::FrVect>     frvect_pointer;
#else
typedef FrameCPP::FrameH*     frame_pointer;
typedef FrameCPP::FrRawData*  frrawdata_pointer;
typedef FrameCPP::FrAdcData*  fradcdata_pointer;
typedef FrameCPP::FrVect*     frvect_pointer;
#endif

using namespace std;
using FrameCPP::FrameH;
using FrameCPP::FrAdcData;
using FrameCPP::FrRawData;
using FrameCPP::FrProcData;
using FrameCPP::FrVect;
using FrameCPP::IFrameStream;

const char*
str_data_type(frvect_pointer v) {
    int  vtype = v->GetType();
    switch (vtype) {
    case FrVect::FR_VECT_2S:
    case FrVect::FR_VECT_2U:
        return "int_2";
    case FrVect::FR_VECT_4S:
    case FrVect::FR_VECT_4U:
	return "int_4";
    case FrVect::FR_VECT_4R:
        return "real_4";
    case FrVect::FR_VECT_8R:
        return "real_8";
    case FrVect::FR_VECT_8C:
        return "complex_8";
    case FrVect::FR_VECT_16C:
        return "complex_16";
    default:
        break;
    }
    return "unknown";
}

//======================================  Frame comparison main function
int
main(int argc, const char* argv[]) {
    //----------------------------------  Parse the command line arguments
    int  verbose = 0;
    bool ratemsg = false;
    bool print_hdr = true;
    int  access_lvl = 0;
    bool syntax  = false;
    for (int i=1 ; i<argc-1 ; i++) {
        string argi = argv[i];
        if (argi == "-a") {
	    access_lvl = strtol(argv[++i], 0, 0);
	} else if (argi == "-h") {
	    print_hdr = false;
	} else if (argi == "-v") {
	    verbose++;
	} else {
	    syntax = true;
	    cerr << "Unrecognized argument: " << argi << endl;
	}
    }

    //-----------------------------------  Print syntax and bail on error/help
    if (syntax || argc < 2 || *argv[argc-1] == '-') { 
        cerr << "Command Syntax: " << endl;
	cerr << "build-channel-list [-v] <file>" << endl;
	return 1;
    }

    //-----------------------------------  Parse the file name
    string file(argv[argc-1]);
    string::size_type inxm = file.rfind("/");
    string sourcename;
    if (inxm == string::npos) sourcename = file;
    else                      sourcename = file.substr(inxm+1);
    inxm = sourcename.find('-', 0);
    if (inxm < string::npos) inxm = sourcename.find('-', inxm+1);
    if (inxm < string::npos) sourcename = sourcename.erase(inxm);

    FrameCPP::Common::FrameBuffer< General::filebuf >* 
	fbuf = new FrameCPP::Common::FrameBuffer<General::filebuf>(ios::in);
    if (!fbuf->open(file.c_str(), ios::in)) {
	cerr << "Unable to open frame file: " << file << endl;
	return 2;
    }
    IFrameStream ifstr(fbuf);
    frame_pointer frame(ifstr.ReadNextFrame());

    //----------------------------------  Build a list of FrAdcData channels
    frrawdata_pointer raw(frame->GetRawData());
    if (!raw) {
	if (verbose) cout << "File has no raw data." << endl;
    } else {
	FrRawData::const_firstAdc_iterator adcIter=raw->RefFirstAdc().begin();
	for ( ; adcIter != raw->RefFirstAdc().end() ; adcIter++ ) {
	    fradcdata_pointer ap(*adcIter);
	    string name = ap->GetName();
	    string type = "raw";
	    double rate = ap->GetSampleRate();
	    frvect_pointer    vp = ap->RefData().front();

	    //--------------------------  Test the rate field against dx
	    double rdxprod = rate * vp->GetDim(0).GetDx();
	    if (fabs(rdxprod - 1.0) > 1e-6) {
		if (!ratemsg) {
		    cerr << "Sample rate inconsistent for channel: " << name
			 << " in file: " << file << endl;
		    ratemsg = true;
		}
		rate = 1.0/vp->GetDim(0).GetDx();
	    }

	    //--------------------------  Look for trend subchannel name.
	    string::size_type inx = name.find_last_of(".");
	    if (inx != string::npos) {
		string ext = name.substr(inx);
		if (ext == ".n" || ext == ".mean" || ext == ".rms" ||
		    ext == ".min" || ext == ".max") {
		    if (rate == 1.0) {
			type = "s-trend";
		    } else {
			type = "m-trend";
		    }
		}
	    }

	    //--------------------------  Print the channel name.
	    cout << name << " " << type << " " << access_lvl << " " 
		 << str_data_type(vp) << " " << rate << " " 
		 << sourcename << endl;
	}
    }

    //----------------------------------  Build a list of FrProcData channels
    for (FrameH::const_procData_iterator i=frame->RefProcData().begin();
	 i != frame->RefProcData().end(); i++) {
	frvect_pointer vp = (*i)->RefData().front();
	double rate = 1./vp->GetDim(0).GetDx();
	cout << (*i)->GetName() << " reduced " << access_lvl << " "
	     << str_data_type(vp) << " " << rate << " " 
	     << sourcename << endl;
    }
#ifndef FCPP_SHARED_PTRS
    delete frame;
#endif
    return 0;
}
